/*-------------------------------------------------------------*/
/* Ansteuerung des Fischer-Technik Interface C64/VC20 ueber    */
/* die CENTRONICS-Schnittstelle, IBM-(kompat)-PC               */
/*                                                             */
/*                                                             */
/* Kabelbelegung (ft-Interface <-> Centronics-Stecker):        */
/* 10-adriges ft-Interface Kabel, BLAU ist Ader1:              */
/*                                                             */
/*-------------------------------------------------------------*/
/* ft-Kabel!CENTRONICS                     ! fischertechnik    */
/* Ader    !PIN!Funktion     Abfrage       ! INTERFACE         */
/*         !   !             (STATUSPORT)  !                   */
/*-------------------------------------------------------------*/
/*   1     !18 !GROUND                     ! Masse             */
/*   2     ! 2 !DATA 1                     ! LOAD-OUT     C    */
/*   3     ! 3 !DATA 2                     ! LOAD-IN      D    */
/*   4     ! 4 !DATA 3                     ! DATA-OUT     E    */
/*   5     ! 5 !DATA 4                     ! CLOCK        F    */
/*   6     !13 !SLCT          & 16         ! COUNT-IN     6    */
/*   7     ! 6 !DATA 5                     ! TRIGGER-X    H    */
/*   8     ! 7 !DATA 6                     ! TRIGGER-Y    J    */
/*   9     !   !                           ! Takt ?            */
/*  10     !12 !PAPER END     & 32         ! DATA-IN      L    */
/*-------------------------------------------------------------*/
/*-- 231193 TOPM Create                                        */
/*-- 291198 TOPM WIN32                                         */
/*-- 061298 TOPM Analogeingang anders auswerten                */
/*-------------------------------------------------------------*/
/* Dieses Modul enthaelt eine Reihe von LOW-LEVEL-Funktionen,  */
/* um das fischertechnik-Computing-Interface C64/VC20          */
/* (Art.Nr. 30 566) ueber die parallele Schnittstelle eines    */
/* IBM-PCs anzusteuern.                                        */
/* Zu diesem Zweck muss der PC-seitige Stecker des Interface   */
/* durch einen CENTRONICS-Stecker ersetzt werden.              */
/* Die o.a. Tabelle gibt die Kabelbelegung an.                 */
/* Die PIN-Angaben beziehen sich auf die Belegung der          */
/* CENTRONICS-Schnittstelle. Die INTERFACE-Angaben auf die     */
/* Belegung des ft-Interface.                                  */
/*                                                             */
/* Literatur:                                                  */
/*                                                             */
/* Elsing,J., Wiencek, A.: Schnittstellen-Handbuch,Mnchen,1989*/
/* fischertechnik-computing, 2.Aufl., fischerwerke, 1988       */
/*                                                             */
/* Hinweis:                                                    */
/*                                                             */
/* Das Modul habe ich fuer meinen "Hausgebrauch" geschrieben.  */
/* Ansprueche auf Fehlerfreiheit, optimale Realisierung oder   */
/* allgemeine Verwendbarkeit sind daher nicht zu erheben.      */
/*                                                             */
/* Autor:                                                      */
/*                                                             */
/* Wolfgang Topmann                                            */
/* email: topmann@t-online.de                                  */
/*-------------------------------------------------------------*/
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include <time.h>
#include "ft.h"

int setbit(int wert, int bit);
int clbit (int wert, int bit);

/*------------------------------------------------------------*/
/*    Kompatible Anweisungen zur DOKU                         */
/*------------------------------------------------------------*/
/*------------------------------------------------------------*/
/*    Motor 1 schrittweise vor (Abfrage auf Eingang 2)        */
/*------------------------------------------------------------*/
void X1V(struct FISCH *fischptr)
{
LDE;
while (E2 == EIN)    /* Schalter bei Anlauf noch geschlossen */
   {
   L1L;
   LDE;
   };
while (E2 == AUS)    /* Bis zum naechsten Schaltersignal weiter */
   {
   L1L;
   LDE;
   };
L1A;
}

/*------------------------------------------------------------*/
/*    Motor 1 schrittweise zurueck (Abfrage auf Eingang 2)    */
/*------------------------------------------------------------*/
void X1Z(struct FISCH *fischptr)
{
LDE;
while (E2 == EIN)    /* Schalter bei Anlauf noch geschlossen */
   {
   L1R;
   LDE;
   };
while (E2 == AUS)    /* Bis zum naechsten Schaltersignal weiter */
   {
   L1R;
   LDE;
   };
L1A;
}

/*------------------------------------------------------------*/
/*    Motor 2 schrittweise vor (Abfrage auf Eingang 3)        */
/*------------------------------------------------------------*/
void X2V(struct FISCH *fischptr)
{
LDE;
while (E3 == EIN)    /* Schalter bei Anlauf noch geschlossen */
   {
   L2L;
   LDE;
   };
while (E3 == AUS)    /* Bis zum naechsten Schaltersignal weiter */
   {
   L2L;
   LDE;
   };
L2A;
}

/*------------------------------------------------------------*/
/*    Motor 2 schrittweise zurueck (Abfrage auf Eingang 3)    */
/*------------------------------------------------------------*/
void X2Z(struct FISCH *fischptr)
{
LDE;
while (E3 == EIN)    /* Schalter bei Anlauf noch geschlossen */
   {
   L2R;
   LDE;
   };
while (E3 == AUS)    /* Bis zum naechsten Schaltersignal weiter */
   {
   L2R;
   LDE;
   };
L2A;
}

/*------------------------------------------------------------*/
/*    Motor 1 links                                           */
/*------------------------------------------------------------*/
void X1L(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m1 = 'L';
senden(fischptr);
}

/*------------------------------------------------------------*/
/*    Motor 1 rechts                                          */
/*------------------------------------------------------------*/
void X1R(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m1 = 'R';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 1 anhalten                                        */
/*------------------------------------------------------------*/
void X1A(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m1 = 'N';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 2 links                                           */
/*------------------------------------------------------------*/
void X2L(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m2 = 'L';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 2 rechts                                          */
/*------------------------------------------------------------*/
void X2R(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m2 = 'R';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 2 anhalten                                        */
/*------------------------------------------------------------*/
void X2A(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m2 = 'N';
senden(fischptr);
}

/*------------------------------------------------------------*/
/*    Motor 3 links                                           */
/*------------------------------------------------------------*/
void X3L(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m3 = 'L';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 3 rechts                                          */
/*------------------------------------------------------------*/
void X3R(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m3 = 'R';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 3 anhalten                                        */
/*------------------------------------------------------------*/
void X3A(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m3 = 'N';
senden(fischptr);
}

/*------------------------------------------------------------*/
/*    Motor 4 links                                           */
/*------------------------------------------------------------*/
void X4L(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m4 = 'L';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 4 rechts                                          */
/*------------------------------------------------------------*/
void X4R(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m4 = 'R';
senden(fischptr);
}
/*------------------------------------------------------------*/
/*    Motor 4 anhalten                                        */
/*------------------------------------------------------------*/
void X4A(struct FISCH *fischptr)
{
void senden    (struct FISCH *fischptr);

fischptr->m4 = 'N';
senden(fischptr);
}

/*------------------------------------------------------------*/
/*    Digitaleingaenge lesen                                  */
/*------------------------------------------------------------*/
void XDE(struct FISCH *fischptr)
{
void empfangen (struct FISCH *fischptr);
empfangen(fischptr);
}

/*------------------------------------------------------------*/
/*    Digitaleingaenge lesen                                  */
/*------------------------------------------------------------*/
void XIN(struct FISCH *fischptr)
{
void analog1 (struct FISCH *fischptr);
void analog2 (struct FISCH *fischptr);
analog1(fischptr);
analog2(fischptr);
}

/*------------------------------------------------------------*/
void senden(struct FISCH *fischptr)

{
int ausgabe;
int i;
int setbit(int wert, int bit);
void output(int port, int wert);
int trans(char m, int wert, int byte1, int byte2);
int wert;
wert    = 0;
ausgabe = 0xFF;

wert = trans(fischptr->m1, wert, 1 , 2);
wert = trans(fischptr->m2, wert, 3 , 4);
wert = trans(fischptr->m3, wert, 5 , 6);
wert = trans(fischptr->m4, wert, 7 , 8);


/* Schritt 1: LOAD-OUT auf LOW legen. DATA 1 */
ausgabe = clbit(ausgabe, 1);

/* Schritt 2: Schleife, Daten fuettern */
for (i = 1; i < 9; i++)
   {
   if (wert & 128)
      ausgabe = setbit(ausgabe, 3);
   else
      ausgabe = clbit (ausgabe, 3);
   wert = wert << 1;
   /* Clock rauf, Clock runter um Daten zu uebernehmen */
   ausgabe = clbit (ausgabe, 4);

   output(DATAPORT, ausgabe);
   ausgabe = setbit(ausgabe, 4);

   output(DATAPORT, ausgabe);
   if (i < 8)
       ausgabe = clbit (ausgabe, 4);
   else
      ausgabe = setbit(ausgabe, 1);

   output(DATAPORT, ausgabe);
   };
/*sleep( (clock_t) 0.001 * CLOCKS_PER_SEC);*/ /* Zeit lassen f. Mechanik */
for(i=1; i<500; i++);
ausgabe = setbit(ausgabe, 1);
}

/*------------------------------------------------------------*/
void empfangen(struct FISCH *fischptr)

{
int ausgabe = 0;
int i, temp;
int setbit(int wert, int bit);
void showbit(int wert);
void output(int port, int wert);

/* ausgabe = setbit (ausgabe, 1);*/  /* LOAD-OUT HIGH        */

ausgabe = setbit(ausgabe, 2);   /* LOAD-IN HIGH  (DATA 2) */
ausgabe = clbit (ausgabe, 3);   /* DATA-OUT LOW  (DATA 3) */
ausgabe = clbit (ausgabe, 4);   /* CLOCK   LOW   (DATA 4) */
output(DATAPORT, ausgabe);
/* Schritt 2: Schleife, Daten lesen */
for (i = 7; i > -1; i--)
   {
   /* Daten lesen */
   /* Clock rauf, Clock runter um Daten zu uebernehmen */
   ausgabe = setbit(ausgabe, 4);
   output(DATAPORT, ausgabe);
   if (i == 7)
      ausgabe = clbit(ausgabe,2);

   /*-- 031096 --*/
   ausgabe = clbit(ausgabe, 4);
   output(DATAPORT, ausgabe);

   temp = _inp(STATUSPORT);
   if (temp & 32)
      fischptr->schalter[i] = '1';
   else
      fischptr->schalter[i] = '0';

   /*-- 031096
   ausgabe = clbit(ausgabe, 4);
   output(DATAPORT, ausgabe);
   --*/
   };
}

/*------------------------------------------------------------*/
void analog1(struct FISCH *fischptr)

{
int analasm(short portein, short portaus);
fischptr->analog[0] = analasm(251,225);
}

/*------------------------------------------------------------*/
void analog2(struct FISCH *fischptr)

{
int analasm(short portein, short portaus);
fischptr->analog[1] = analasm(251,209);
}




/*------------------------------------------------------------*/
int analasm(short portein, short portaus)

{
short max   = 5000;     /* TIMEOUT-Zaehler */
short j   = 0;
int t     = 0;
/*-- Problem: Auf COUNT-IN liegt ein Rechtecksignal, dessen   --*/
/*-- Flanken eigentlich gezaehlt werden muessten. Leider      --*/
/*-- reicht die Auflsung nicht aus. Insbesondere wenn andere --*/
/*-- Tasks auf dem Rechner aktiv sind, koennen abhaengig davon--*/
/*-- die Ergebnisse stark schwanken, weil waehrend der Dauer  --*/
/*-- des Rechtecksignals diesem Prozess mehr oder weniger     --*/
/*-- Rechenzeit zugeteilt wird. Thats Life :-(                --*/
__asm
   {
        push dx;
                push ax;
                push bx;
                cli                      ; DISABLE INTERRUPT
                mov dx,DATAPORT  ; TRIGGERN
                mov ax,portein
                out dx,ax
                mov ax,portaus
                out dx,ax
                mov ax,portein
                out dx,ax
                mov dx,STATUSPORT
                mov bx,0     ; Zaehler fuer Flankenwechsel
NEXT:   inc  j
                mov  ax,max
                cmp  ax,j
                jz   ENDE        ; TIME-OUT-Schwelle erreicht
                in   al,dx       ; STATUSPORT lesen
                and  ax,16       ; COUNT-IN pruefen
                jz   HOCH        ; NULL: COUNT-IN HIGH
        jmp  UNTEN   ;       COUNT-IN LOW
HOCH:   cmp  bx,1
        jz   NEXT    ; Auch im vorhergehenden Durchlauf war der Wert HIGH
                mov  bx,1    ; Und dann setzen wir den Merker auf 1
                jmp  NEXT
UNTEN:  cmp  bx,0
        jz   NEXT    ; Auch im vorherigen Durchlauf war der Wert LOW
        mov  bx,0    ; Wenn wir hier landen, war ein Signalwechsel
                inc  t       ; Wenn wir hier landen, war ein Signalwechsel
                jmp  NEXT
ENDE:   pop  bx;
                pop  ax;
                pop  dx;
        sti                  ; ENABLE INTERRUPT
   } /* __asm Ende */
return(t);
}

/*------------------------------------------------------------*/
void xanalog1(struct FISCH *fischptr)

{
int ausgabe;
int i, temp;
int setbit(int wert, int bit);
void output(int port, int wert);

ausgabe = 0xFF;

fischptr->analog[0] = 0;

__asm cli
ausgabe = 251;
output(DATAPORT, ausgabe);
ausgabe = 225;
output(DATAPORT, ausgabe);
ausgabe = 251;
output(DATAPORT, ausgabe);
__asm cli
for (i = 1; i < 1000; i++)
   {
   temp = _inp(STATUSPORT);
   if ( ! (temp & 16))
      {
      fischptr->analog[0]++;
      }
   };
__asm sti

return;

ausgabe = clbit (ausgabe, 3);   /* DATA-OUT LOW  (DATA 3) */
ausgabe = setbit(ausgabe, 2);   /* LOAD-IN  HIGH (DATA 2) */
/* Load-in muss HIGH sein, sonst keine Abfrage ! */
ausgabe = setbit(ausgabe, 1);   /* LOAD-OUT HIGH (DATA 1) */


ausgabe = setbit(ausgabe, 2);   /* LOAD IN HIGH  (DATA 2) */
ausgabe = setbit(ausgabe, 5);   /* TRIGGER HIGH  (DATA 5) */
ausgabe = setbit(ausgabe, 4);   /* CLOCK   HIGH  (DATA 4) */
output(DATAPORT, ausgabe);

ausgabe = clbit (ausgabe, 2);   /* LOAD IN LOW   (DATA 2) */
ausgabe = clbit (ausgabe, 5);   /* TRIGGER LOW   (DATA 5) */
ausgabe = clbit (ausgabe, 4);   /* CLOCK   LOW   (DATA 4) */
output(DATAPORT, ausgabe);

ausgabe = setbit(ausgabe, 2);   /* LOAD IN HIGH  (DATA 2) */
ausgabe = setbit(ausgabe, 5);   /* TRIGGER HIGH  (DATA 5) */
ausgabe = setbit(ausgabe, 4);   /* CLOCK   HIGH  (DATA 4) */
output(DATAPORT, ausgabe);
__asm cli
for (i = 1; i < 1000; i++)
   {
   temp = _inp(STATUSPORT);
   if ( ! (temp & 16))
      {
      fischptr->analog[0]++;
      }
   };
__asm sti
}


/*------------------------------------------------------------*/
void xanalog2(struct FISCH *fischptr)

{
int ausgabe;
int i, temp;
int setbit(int wert, int bit);
void output(int port, int wert);

ausgabe = 0xFF;

fischptr->analog[1] = 0;

__asm cli
ausgabe = 251;
output(DATAPORT, ausgabe);
ausgabe = 209;
output(DATAPORT, ausgabe);
ausgabe = 251;
output(DATAPORT, ausgabe);
__asm cli
for (i = 1; i < 1000; i++)
   {
   temp = _inp(STATUSPORT);
   if ( ! (temp & 16))
      {
      fischptr->analog[1]++;
      }
   };
__asm sti

return;

ausgabe = clbit (ausgabe, 3);   /* DATA-OUT LOW  (DATA 3) */
ausgabe = setbit(ausgabe, 2);   /* LOAD-IN  HIGH (DATA 2) */
/* Load-in muss HIGH sein, sonst keine Abfrage ! */

ausgabe = setbit(ausgabe, 1);   /* LOAD-OUT HIGH (DATA 1) */


ausgabe = setbit(ausgabe, 2);   /* LOAD IN HIGH  (DATA 2) */
ausgabe = setbit(ausgabe, 6);   /* TRIGGER HIGH  (DATA 5) */
ausgabe = setbit(ausgabe, 4);   /* CLOCK   HIGH  (DATA 4) */
output(DATAPORT, ausgabe);

ausgabe = clbit (ausgabe, 2);   /* LOAD IN LOW   (DATA 2) */
ausgabe = clbit (ausgabe, 6);   /* TRIGGER HIGH  (DATA 5) */
ausgabe = clbit (ausgabe, 4);   /* CLOCK   LOW   (DATA 4) */
output(DATAPORT, ausgabe);

ausgabe = setbit(ausgabe, 2);   /* LOAD IN HIGH  (DATA 2) */
ausgabe = setbit(ausgabe, 6);   /* TRIGGER HIGH  (DATA 5) */
ausgabe = setbit(ausgabe, 4);   /* CLOCK   HIGH  (DATA 4) */
output(DATAPORT, ausgabe);

__asm cli
for (i = 1; i < 1000; i++)
   {
   temp = _inp(STATUSPORT);
   if (! (temp & 16) )
      {
      fischptr->analog[1]++;
      }
   };
__asm sti
}

/*------------------------------------------------------------*/
void output(int port, int wert)
{

_outp(DATAPORT,wert);
}

/*------------------------------------------------------------*/
/*  Eingabewert umsetzen                                      */
/*------------------------------------------------------------*/
int trans(char m, int wert, int byte1, int byte2)

{

if (m == 'L')
   {
   wert = setbit(wert,byte1);
   wert = clbit (wert,byte2);
   }  else
if (m == 'R')
   {
   wert = setbit(wert,byte2);
   wert = clbit (wert,byte1);
   }  else
if (m == 'N')
   {
   wert = clbit (wert,byte1);
   wert = clbit (wert,byte2);
   };
return(wert);
}

/*------------------------------------------------------------*/
/*  Bit setzen                                                */
/*------------------------------------------------------------*/
int setbit(int wert, int bit)

{
switch (bit)
   {
   case 1:
      wert = wert |   1;
      break;
   case 2:
      wert = wert |   2;
      break;
   case 3:
      wert = wert |   4;
      break;
   case 4:
      wert = wert |   8;
      break;
   case 5:
      wert = wert |  16;
      break;
   case 6:
      wert = wert |  32;
      break;
   case 7:
      wert = wert |  64;
      break;
   case 8:
      wert = wert | 128;
      break;
   default:
      exit(99);
      break;
   };
return(wert);
}

/*------------------------------------------------------------*/
/*  Bit loeschen                                              */
/*------------------------------------------------------------*/
int clbit(int wert, int bit)

{
switch (bit)
   {
   case 1:
      wert = wert &   254;
      break;
   case 2:
      wert = wert &   253;
      break;
   case 3:
      wert = wert &   251;
      break;
   case 4:
      wert = wert &   247;
      break;
   case 5:
      wert = wert &   239;
      break;
   case 6:
      wert = wert &   223;
      break;
   case 7:
      wert = wert &   191;
      break;
   case 8:
      wert = wert &   127;
      break;
   default:
      exit(99);
      break;
   };
return(wert);
}

/*------------------------------------------------------------*/
/* Struktur initialisieren                                    */
/*------------------------------------------------------------*/
void finit (struct FISCH *fischptr)

{
int i;

for (i = 0; i < 2; i++)
   {
   fischptr->m1 = '0';
   fischptr->m2 = '0';
   fischptr->m3 = '0';
   fischptr->m4 = '0';
   };

for (i = 0; i < 8; i++)
   {
   fischptr->schalter[i] = '0';
   };

fischptr->analog[0] = 0;
fischptr->analog[1] = 0;

/* Ausgaenge auf definierten Status setzen */
L1A;
L2A;
L3A;
L4A;
}

/*------------------------------------------------------------*/
/* Zeitschleife                                               */
/*------------------------------------------------------------*/
void sleep (clock_t wait)

{
long int i,k;
double erg;

for (k = 0; k < wait; k++)
   for (i = 0; i < 500; i++)
      erg = 2.2122 * 41.5858;

/*
goal = wait + clock();
while (goal > clock() );
*/
}


